#!/usr/bin/env python
    

import sys, string, os, copy, EM, re
from qt import *
from gui_main import *
from gui_ctf import *
from gui_parallel import *
from gui_reconstruction import *
from gui_masking import *
from gui_preview import *



fileopen = [
    '16 13 5 1',
    '. c #040404',
    '# c #808304',
    'a c None',
    'b c #f3f704',
    'c c #f3f7f3',
    'aaaaaaaaa...aaaa',
    'aaaaaaaa.aaa.a.a',
    'aaaaaaaaaaaaa..a',
    'a...aaaaaaaa...a',
    '.bcb.......aaaaa',
    '.cbcbcbcbc.aaaaa',
    '.bcbcbcbcb.aaaaa',
    '.cbcb...........',
    '.bcb.#########.a',
    '.cb.#########.aa',
    '.b.#########.aaa',
    '..#########.aaaa',
    '...........aaaaa'
]

filesave = [
    '14 14 4 1',
    '. c #040404',
    '# c #808304',
    'a c #bfc2bf',
    'b c None',
    '..............',
    '.#.aaaaaaaa.a.',

    '.#.aaaaaaaa...',
    '.#.aaaaaaaa.#.',
    '.#.aaaaaaaa.#.',
    '.#.aaaaaaaa.#.',
    '.#.aaaaaaaa.#.',
    '.##........##.',
    '.############.',
    '.##.........#.',
    '.##......aa.#.',
    '.##......aa.#.',
    '.##......aa.#.',
    'b.............'
]

#editorList = []
SAVRDIR='.'
version = '0.9b'
title ='Semi Automated Virus Reconstruction. Version '+version
DoublePattern = r"([0-9,.]+)"
DoubleRegExp =  re.compile(DoublePattern)


FALSE = 0
TRUE = 1

def createString(object,fmt=""):
    try:
	if fmt: 
	    objectstring=fmt % object
	else: objectstring=str(object)
    except:
        #print "Error creating string: ",object," :"
        return ""
    return objectstring

def createFloat(object):
    try:
        objectfloat=float(createString(object))
    except:
        #print "Error creating a float:", object," :"
        return 0.0
    return objectfloat

def createInt(object):
    try:
        objectint=int(createString(object))
    except:
        return 0
    return objectint

class PreviewDialog(TextDisplayDialog):
    def __init__(self,parent=None,text="Insert Text Here"):
        TextDisplayDialog.__init__(self,parent)
        #print text
        #print self
        Qtext=QString(text)
        #print Qtext
        self.TextView.setText(Qtext)
        #print "finished"

class EMRunAdvDialog(QTabDialog):
    def __init__(self,project,parent=None):
        QTabDialog.__init__(self,parent,"Advanced Controls")
        
        self.proj=project   
        self.ctfcorr=CTFCorrAdv(self)
        self.recon=ReconPropAdv(self)
        self.mask=MaskingAdv(self)
        self.parallel=Parallel(self)
        self.addTab(self.recon,"3-D Reconstruction And Refinement")
        self.addTab(self.ctfcorr,"CTF Correction")
        self.addTab(self.mask,"Masking")
        self.addTab(self.parallel,"Parallelization")
        self.setCancelButton()
        
        self.connect(self,SIGNAL("applyButtonPressed()"),self.updateProj)
        self.connect(self.recon.pbSFFileBrowse,SIGNAL("clicked()"),self.SFFileBrowseClicked)
        
        
        #Set Validators
        self.recon.leRefineMore.setValidator(QDoubleValidator(self))
        self.recon.leCCMLPLimits0.setValidator(QDoubleValidator(self))
        self.recon.leCCMLPLimits1.setValidator(QDoubleValidator(self))
        self.recon.leDefocusRange0.setValidator(QDoubleValidator(self))
        self.recon.leDefocusRange1.setValidator(QDoubleValidator(self))
        self.recon.leDeepTTestRes.setValidator(QDoubleValidator(self))
        self.recon.leDeepTTestLevel.setValidator(QIntValidator(self))
        self.recon.leRefineCycleSizeHigh.setValidator(QIntValidator(self))
        self.recon.leRefineCycleSizeLow.setValidator(QIntValidator(self))
        self.recon.leWAVECYCLES.setValidator(QIntValidator(self))
        self.recon.leFixedBFactor.setValidator(QDoubleValidator(self))
        
        self.mask.leICFactor.setValidator(QDoubleValidator(self))
        self.mask.leMASKW.setValidator(QDoubleValidator(self))
        
        self.loadProj()
        
    def SFFileBrowseClicked(self):
        fn = QFileDialog.getOpenFileName(QString.null,"",self)
        if fn.isEmpty():
            self.recon.leSFFile.validateAndSet('',0,0,0)
            return
        fileName = createString(fn)
        #print fileName
        self.recon.leSFFile.validateAndSet(fileName,0,0,0)
         
    def loadProj(self):
        self.recon.leRefineMore.validateAndSet(createString(self.proj.RefineMore),0,0,0)
        self.recon.leDeepTTestLevel.validateAndSet(createString(self.proj.DeepTTestLevel),0,0,0)
        self.recon.leCCMLPLimits0.validateAndSet(createString(self.proj.CCMLPLimits[0]),0,0,0)
        self.recon.leCCMLPLimits1.validateAndSet(createString(self.proj.CCMLPLimits[1]),0,0,0)
        self.recon.leDeepTTestRes.validateAndSet(createString(self.proj.DeepTTestRes),0,0,0)
        try:
            self.recon.leDefocusRange0.validateAndSet(createString(self.proj.DefocusRange[0]),0,0,0)
            self.recon.leDefocusRange1.validateAndSet(createString(self.proj.DefocusRange[1]),0,0,0)
        except IndexError:
            self.recon.leDefocusRange0.validateAndSet("",0,0,0)
            self.recon.leDefocusRange1.validateAndSet("",0,0,0)
        try:
	    self.recon.leRefineCycleSizeLow.validateAndSet(createString(self.proj.RefineCycleSize[0]),0,0,0)
	    self.recon.leRefineCycleSizeHigh.validateAndSet(createString(self.proj.RefineCycleSize[1]),0,0,0)
        except IndexError:
	    self.recon.leRefineCycleSizeLow.validateAndSet("",0,0,0)
	    self.recon.leRefineCycleSizeHigh.validateAndSet("",0,0,0)
	
	self.recon.leSFFile.validateAndSet(createString(self.proj.SFFile),0,0,0)
        self.recon.leWAVECYCLES.validateAndSet(createString(self.proj.WaveCycles),0,0,0)
        self.recon.cbIgnoreFocalPair.setChecked(TRUE)        
        self.recon.cbIgnoreFocalPair.setEnabled(FALSE)
        self.recon.cbIgnoreSrcImgs.setChecked(self.proj.IgnoreSrcImgs)

        self.recon.cbFixedBFactor.setChecked(self.proj.FixedBFactor)
        self.recon.leFixedBFactor.validateAndSet(createString(self.proj.FixedBFactor),0,0,0)
        
        self.recon.cbCenAlignOpt.setChecked(self.proj.CenAlignOpt)
        self.recon.leSFFile.validateAndSet(createString(self.proj.SFFile),0,0,0)
        self.recon.cbCCMLProjFactor.setChecked(self.proj.CCMLProjFactor)
        self.recon.cbMultSampling.setChecked(self.proj.MultSampling)
        self.recon.cbWaveProjFactor.setChecked(self.proj.WaveProjFactor)
        
        MRCRefineString=""
        for i in range(len(self.proj.MRCRefine)):    
            MRCRefineString=MRCRefineString+" "+createString(self.proj.MRCRefine[i])
        self.recon.leMRCRefine.validateAndSet(MRCRefineString,0,0,0)  
        self.mask.leICFactor.validateAndSet(createString(self.proj.ICFactor),0,0,0)
        self.mask.leMASKW.validateAndSet(createString(self.proj.IMaskSlope),0,0,0)
        
        self.ctfcorr.cbCTFCorrect.setChecked(self.proj.CTFCorrect)
        self.ctfcorr.cbApplyCTF.setChecked(self.proj.ApplyCTF)
        
        self.mask.cbWaveInMask.setChecked(self.proj.WaveInMask)
        self.mask.cbCCMLInMask.setChecked(self.proj.CCMLInMask)
        try:
            self.mask.cbConvergInMask.setChecked(self.proj.ConvergInMask)
        except:
            self.mask.cbConvergInMask.setChecked(FALSE)
        self.mask.cbSymmetrize.setChecked(self.proj.Symmetrize)
        self.parallel.cbRunPar.setChecked(self.proj.RunPar)
        self.parallel.cbSharedMemory.setChecked(not(self.proj.RunPar))
#         screen
    def updateProj(self):
        #print "Updating project"
        self.proj.RefineMore        =   (createFloat(self.recon.leRefineMore.text()))
        self.proj.DeepTTestLevel    =   (createInt(self.recon.leDeepTTestLevel.text()))
        self.proj.CCMLPLimits    =   [(createFloat(self.recon.leCCMLPLimits0.text())),(createFloat(self.recon.leCCMLPLimits1.text()))]
        self.proj.DeepTTestRes      =   (createFloat(self.recon.leDeepTTestRes.text()))
        try:
            self.proj.DefocusRange   =   [(createFloat(self.recon.leDefocusRange0.text())),(createFloat(self.recon.leDefocusRange1.text()))]
        except ValueError:
            self.proj.DefocusRange  =   []
        self.proj.CCMLProjFactor    =   self.recon.cbCCMLProjFactor.isChecked()
        self.proj.MultSampling      =   self.recon.cbMultSampling.isChecked()
        self.proj.WaveProjFactor    =   self.recon.cbWaveProjFactor.isChecked()
        self.proj.SFFile            =   createString(self.recon.leSFFile.text())
        #self.proj.DeepTTestCycle    =   createString(self.recon.leDeepTTestCycle.text())
        self.proj.IgnoreFocalPair   =   self.recon.cbIgnoreFocalPair.isChecked()
        self.proj.IgnoreSrcImgs     =   self.recon.cbIgnoreSrcImgs.isChecked()
        if(self.recon.cbFixedBFactor.isChecked()):
            self.proj.FixedBFactor = createFloat(self.recon.leFixedBFactor.text())
        self.proj.CenAlignOpt       =   self.recon.cbCenAlignOpt.isChecked()
        self.proj.RefineCycleSize   =   [(createInt(self.recon.leRefineCycleSizeLow.text())),(createInt(self.recon.leRefineCycleSizeHigh.text()))]
           
        self.proj.ICFactor          =   (createFloat(self.mask.leICFactor.text()))
        self.proj.IMaskSlope        =   (createFloat(self.mask.leMASKW.text()))
        try:
            self.proj.WaveCycles        =   (createInt(self.recon.leWAVECYCLES.text()))
        except ValueError:
            self.proj.WaveCycles    = ""
        self.proj.CTFCorrect        =   self.ctfcorr.cbCTFCorrect.isChecked() 
        self.proj.ApplyCTF          =   self.ctfcorr.cbApplyCTF.isChecked()
        self.proj.WaveInMask        =   self.mask.cbWaveInMask.isChecked()
        self.proj.CCMLInMask        =   self.mask.cbCCMLInMask.isChecked()
        self.proj.ConvergInMask     =   self.mask.cbConvergInMask.isChecked()
        self.proj.Symmetrize        =   self.mask.cbSymmetrize.isChecked()
        
        self.proj.RunPar            =   self.parallel.cbRunPar.isChecked()
        self.proj.SharedMemory      =   self.parallel.cbSharedMemory.isChecked()
        
        MRCRefineString = createString(self.recon.leMRCRefine.text())
        MRCRefineList = DoubleRegExp.findall(MRCRefineString)
        self.proj.MRCRefine=[]
        for i in range(len(MRCRefineList)):
            try:
                self.proj.MRCRefine.append(float(MRCRefineList[i]))
            except ValueError:
                msg = 'There was an error reading [3-D Reconstruction]:[MRC Refinement Steps].\n Please Re-Check These Values'
                QMessageBox.about(self,title,msg)
                print self.proj.MRCRefine
                return
        
        
class EMRunMainForm(Semi_Automated_Virus_Rconstruction):
    
    def __init__(self,parent=None,proj=None):
        Semi_Automated_Virus_Rconstruction.__init__(self,parent)
        self.MicrographTextEntry=TRUE
        self.project = proj
        # Define connections for Push buttons
        self.connect( self.pbStartModelBrowse, SIGNAL("clicked()"), self.pbStartModelBrowseClicked )
        self.connect( self.pbProjBrowse, SIGNAL("clicked()"), self.pbProjBrowseClicked )
        self.connect(self.pbAdvancedSettings, SIGNAL("clicked()"),self.pbAdvancedSettingsClicked)
        
        #Set Movie
        #MOVIEFILE = "savr.gif"
        #self.movie = QMovie( MOVIEFILE )
        #self.movie.setSpeed(500)
        #self.movie.step(59)
        #self.MovieLabel.setMovie(self.movie)

	IMAGEFILE = SAVRDIR+'/savr.png'
	#print 'IMAGEFILE: ', IMAGEFILE
	pixmap=QPixmap(IMAGEFILE)
	self.MovieLabel.setPixmap(pixmap)
	
	
        #Set Validators
        self.leNumCPU.setValidator(QIntValidator(self))    
        self.leMass.setValidator(QDoubleValidator(self))
        self.leParticleSize.setValidator(QIntValidator(self))
        self.leInnerRadius.setValidator(QIntValidator(self))
        self.leProjStartModelRes.setValidator(QDoubleValidator(self))
        self.leSMPixelSize.setValidator(QDoubleValidator(self))
        self.leResLow.setValidator(QDoubleValidator(self))
        self.leResHigh.setValidator(QDoubleValidator(self))
        self.lePixSize.setValidator(QDoubleValidator(self))
        self.leMapSize.setValidator(QIntValidator(self))
        self.leCs.setValidator(QDoubleValidator(self))
        self.leVolt.setValidator(QDoubleValidator(self))
        self.leAmpCont.setValidator(QDoubleValidator(self))

        self.connect(self.TableAddMicrograph,SIGNAL("clicked()"),self.addMicrograph)
        self.connect(self.TableDelMicrograph,SIGNAL("clicked()"),self.delMicrograph)
	self.connect(self.cbVerbose,SIGNAL("clicked()"),self.updateVerbose)
        self.Table1.setNumCols(8)  
        #self.loadProj()

        #self.Table1.setSorting(TRUE)
        
        self.horizHeader=self.Table1.horizontalHeader()
        self.horizHeader.setLabel(0,"Micrograph Location")
        self.horizHeader.setLabel(1,"Angstroms/Pixel")
        self.horizHeader.setLabel(2,"Defocus (micron)")
        self.horizHeader.setLabel(3,"B-Factor")
        self.horizHeader.setLabel(4,"CTF First Peak")
        self.horizHeader.setLabel(5,"CTF First Zero")
        self.horizHeader.setLabel(6,"Number Of Particles")
        self.horizHeader.setLabel(7,"Dimension of Particle Images")
        #self.LCDNumTotalImages = 0
        #self.vertHeader=self.Table1.verticalHeader()
        self.loadProj(proj)
        self.Table1.adjustColumn(0)
        #self.Table1.setRowMovingEnabled(TRUE)
        
#    def focusInEvent(self,event):
#            print event
        #if self.movie.paused():
#        self.movie.unpause()
#        self.movie.restart()
#        print self.movie.speed()
 #       self.movie.step(59) #one loop

#    def focusOutEvent(self,event):
#            print event
#            self.movie.pause()

    def updateVerbose(self):
	self.project.Verbose		=	self.cbVerbose.isChecked()

    def addMicrograph(self):
        fn = QFileDialog.getOpenFileNames(QString.null,"Img Files (*.img)",self)
        if fn.isEmpty():
            #self.leMicrographLocation.validateAndSet('',0,0,0)
            return
#        rows=self.Table1.numRows()
	for fileName in fn:
            newMicrograph = EM.Micrograph()
            newMicrograph.imagefile=createString(fileName)
	    newMicrograph.ImageInfo()
	    newMicrograph.CtfFirstPeakZero(self.project)
            self.project.Data.append(newMicrograph)    
	self.loadProjTable(self.project)
        self.Table1.adjustColumn(0)
    
    def delMicrograph(self):
        cRow=self.Table1.currentRow()
	
	#Right Now I delete the element from the array and then reload the entire array back into the table
	#This is very slow and should probably be improved for large data sets
	
        self.updateProjTable(self.project)
	try:
	    del self.project.Data[cRow]
	except IndexError:
	    pass
	rows=0
        self.Table1.setNumRows(rows) 
        self.loadProjTable(self.project)
        
    # put proj into form
    def loadProj(self,proj):
        self.project = proj
#        self.leImageSize.validateAndSet(createString(self.project.ImageSize),0,0,0)
        self.leParticleSize.validateAndSet(createString(self.project.ParticleSize),0,0,0)
        self.leInnerRadius.validateAndSet(createString(self.project.InnerRadius),0,0,0)
        self.lePixSize.validateAndSet(createString(self.project.PixelSize),0,0,0)
        self.leVolt.validateAndSet(createString(self.project.Voltage),0,0,0)
        self.leCs.validateAndSet(createString(self.project.Cs),0,0,0)
        self.leAmpCont.validateAndSet(createString(100*self.project.AmpContrast),0,0,0)
        self.leResLow.validateAndSet(createString(self.project.ResRange[0]),0,0,0)
        self.leResHigh.validateAndSet(createString(self.project.ResRange[1]),0,0,0)
        self.leMapSize.validateAndSet(createString(self.project.MapSize),0,0,0)
        self.leProjStartModel.validateAndSet(createString(self.project.StartModel),0,0,0)
        self.leProjDir.validateAndSet(createString(self.project.ProjectRoot),0,0,0)
        self.leProjStartModelRes.validateAndSet(createString(self.project.SMResolution),0,0,0)
        self.leEmail.validateAndSet(createString(self.project.Mail),0,0,0)
        self.leNumCPU.validateAndSet(createString(self.project.CPU),0,0,0)
        self.leSample.validateAndSet(createString(self.project.Sample),0,0,0)
        self.leMass.validateAndSet(createString(self.project.MolMass),0,0,0)

        self.leSMPixelSize.validateAndSet(createString(self.project.SMPixelSize),0,0,0)
        self.cbCTFCorr.setChecked(self.project.SMCTFCorrect)
        self.cbInvertImage.setChecked(self.project.InvertImages)
        self.cbTTest.setChecked(self.project.TTest)
        self.cbVerbose.setChecked(self.project.Verbose)
        self.loadProjTable(self.project)
        
    def loadProjTable(self, proj):
        rows=0
	total=0
        self.Table1.setNumRows(0)
        for i in range(len(proj.Data)):
            proj.Data[rows].ImageInfo()
	    proj.Data[rows].CtfFirstPeakZero(self.project)
	    self.Table1.setNumRows(rows+1)
            self.Table1.setText(rows,0,createString(proj.Data[rows].imagefile))
            self.Table1.setText(rows,1,createString(proj.Data[rows].apix))
            self.Table1.setText(rows,2,createString(proj.Data[rows].defocus))
            self.Table1.setText(rows,3,createString(proj.Data[rows].B))
            self.Table1.setText(rows,4,createString(proj.Data[rows].firstpeak,"%.2f"))
            self.Table1.setText(rows,5,createString(proj.Data[rows].firstzero, "%.2f"))
            self.Table1.setText(rows,6,createString(proj.Data[rows].imagenum))
            self.Table1.setText(rows,7,createString(proj.Data[rows].imagesize))
            total=total+proj.Data[rows].imagenum
	    rows=rows+1
	
	self.LCDNumTotalImages.display(total)
        self.Table1.adjustColumn(0)   

    #Put form data into proj
    def updateProj(self,proj):
        self.project = proj
        self.project.Sample		=	createString(self.leSample.text())
        self.project.MolMass	        =	(createFloat(self.leMass.text()))
        self.project.ProjectRoot 	=	createString(self.leProjDir.text())
        self.project.StartModel	 	=	createString(self.leProjStartModel.text())
        self.project.SMCTFCorrect       =       self.cbCTFCorr.isChecked()
        self.project.SMResolution	=	(createFloat(self.leProjStartModelRes.text()))
        self.project.ResRange[1]	=	(createFloat(self.leResHigh.text()))
        self.project.ResRange[0]        =       (createFloat(self.leResLow.text()))
        self.project.Voltage		=	(createFloat(self.leVolt.text()))
        self.project.Cs			=	(createFloat(self.leCs.text()))
        self.project.AmpContrast	=	(createFloat(self.leAmpCont.text()))/100 
        self.project.InvertImages	=	self.cbInvertImage.isChecked()
        self.project.PixelSize	 	=	(createFloat(self.lePixSize.text()))
        self.project.MapSize	 	=	(createInt(self.leMapSize.text()))
        self.project.ParticleSize	=	(createInt(self.leParticleSize.text()))
        self.project.InnerRadius 	=	(createInt(self.leInnerRadius.text()))
        self.project.CPU		=	(createInt(self.leNumCPU.text()))
        self.project.TTest		=	self.cbTTest.isChecked()
        self.project.Mail		=	createString(self.leEmail.text())
        self.project.Verbose		=	self.cbVerbose.isChecked()
        self.project.SMPixelSize        =       (createFloat(self.leSMPixelSize.text()))
        
        self.updateProjTable(self.project)

    #put only the table data into proj.Data
    def updateProjTable(self,proj):
        #print "updateProjectTable"
	self.project.Data=[]
        for i in range(self.Table1.numRows()):
            newMicrograph = EM.Micrograph()
            newMicrograph.imagefile	=	createString(self.Table1.text(i,0))
            newMicrograph.apix		=	createFloat(self.Table1.text(i,1))    
            newMicrograph.defocus	=	createFloat(self.Table1.text(i,2))
            newMicrograph.B		= 	createFloat(self.Table1.text(i,3))
            newMicrograph.firstpeak	=	createFloat(self.Table1.text(i,4))
            newMicrograph.firstzero	=	createFloat(self.Table1.text(i,5))
            newMicrograph.imagenum	=	(createInt(self.Table1.text(i,6)))
            newMicrograph.imagesize	=	(createInt(self.Table1.text(i,7)))   
            self.project.Data.append(newMicrograph)
        
    def pbStartModelBrowseClicked(self):
        fn = QFileDialog.getOpenFileName(QString.null,"Map Files (*.map)",self)
        if fn.isEmpty():
            #self.leProjStartModel.validateAndSet('',0,0,0)
            return
        fileName = createString(fn)
        #dir = os.path.split(fileName)[0]
        self.leProjStartModel.validateAndSet(fileName,0,0,0)
        self.setEdited(0)
        
    def pbProjBrowseClicked(self): 
        fn = QFileDialog.getExistingDirectory()
        if fn.isEmpty():
            #self.leProjDir.validateAndSet('',0,0,0)
            return
        dir = createString(fn)
        self.leProjDir.validateAndSet(dir,0,0,0)
        self.setEdited(FALSE)
        
        
    def pbAdvancedSettingsClicked(self):
        d=EMRunAdvDialog(project=self.project,parent=self)
        d.show()

    def setEdited(self,value=FALSE):
        for widget in self.__dict__.keys():
            cmd="self.%s.setEdited(%s)" % (createString(widget),value)
            try:
                eval(cmd)
            except:
                pass
        
    def edited(self):
        for widget in self.__dict__.keys():
            cmd="self.%s.edited()" % createString(widget)
            try:
                result=eval(cmd)
                if result==TRUE:
                    return TRUE
            except:
                pass
        return FALSE


class ApplicationWindow(QMainWindow):
    def __init__(self,argv=[]):
        QMainWindow.__init__(self,None,'Application main window',Qt.WType_TopLevel | Qt.WDestructiveClose)
        self.argv=argv
        self.filename = QString.null
        openIcon = QPixmap(fileopen)
        saveIcon = QPixmap(filesave)
        self.data = EM.Project()

        try:
            self.load(sys.argv[1])
        except IndexError,err:
            #print "Error: ",err
            pass
        self.main = EMRunMainForm(self,self.data)
        self.main.setFocus()
        self.setCentralWidget(self.main)
        
        self.file = QPopupMenu(self)
        self.menuBar().insertItem('&File',self.file)

        self.file.insertItem('&New',self.newDoc,Qt.CTRL + Qt.Key_N)
        self.file.insertItem(QIconSet(openIcon),'&Open',self.load,Qt.CTRL + Qt.Key_O)
        self.file.insertItem(QIconSet(saveIcon),'&Save',self.save,Qt.CTRL + Qt.Key_S)
        self.file.insertItem('Save &as',self.saveAs)
        self.file.insertItem('&Close',self,SLOT('close()'),Qt.CTRL + Qt.Key_W)
        self.file.insertItem('&Quit',qApp,SLOT('closeAllWindows()'),Qt.CTRL + Qt.Key_Q)

        self.options=QPopupMenu(self)
        self.options.insertItem('&Advanced Settings...',self.main.pbAdvancedSettingsClicked)
        self.menuBar().insertItem('&Options',self.options)
        
        self.emrun = QPopupMenu(self)
        #self.emrun.insertSeparator()
        self.emrun.insertItem('&Preview Job...',self.previewRun)
        self.emrun.insertItem('&Submit Job',self.startRun)
        self.menuBar().insertItem('&Reconstruction',self.emrun)

        self.help = QPopupMenu(self)
        self.menuBar().insertSeparator()
        self.menuBar().insertItem('&Help',self.help)

        self.help.insertItem('&About',self.about,Qt.Key_F1)
        self.help.insertItem('About &Qt',self.aboutQt)
        

    def previewRun(self):
        self.main.updateProj(self.data)
        prev = PreviewDialog(self,`self.data`)
        prev.show()
        
    def startRun(self):
        try:
            self.main.updateProj(self.data)
            if self.main.edited():
                QMessageBox.about(self,title,"Please save your parameters before you submit the job")
                self.saveAs()
            if not self.main.edited():
		jobid=os.spawnlp(os.P_NOWAIT,'screen','screen','-d','-m',"virus.py",self.filename)
		QMessageBox.about(self,title,"Your job '%d' has been submitted successfully" % (jobid))
        except IndexError, err:
            msg= "There was an error: "+createString(err)
            QMessageBox.warning(self,title,msg)
        except ValueError, err:
            msg="No job has been submitted.\nError: "+createString(err)
            QMessageBox.warning(self,title,msg)
        #self.close()

    def newDoc(self):
        ed = ApplicationWindow()
        ed.show()
        editorList.append(ed)

    def load(self,fileName=None):
        #print fileName
        
        try:
            fileNameToInt=int(fileName)
        except ValueError:
            fileNameToInt=0
         
        if fileName==None or fileNameToInt<0:  #fileNameToInt less than zero means that sys.argv[1] is not set
            fn = QFileDialog.getOpenFileName(QString.null,QString.null,self)
            if fn.isEmpty():
                return
            fileName = createString(fn)
            
        #try:
        #    exec open(fileName).read()  #proj is the instance of the EM.Project() created by this command
        #    self.data=proj
        #    self.main.loadProj(self.data)
        #    self.filename=fn
        #    self.data.ProjectFile=createString(self.filename)
        #    self.main.setEdited(FALSE)
        #except AttributeError:
        #    pass
	#try:
	self.data.ProjectFile=createString(fileName)
	#print self.data.ProjectFile
	self.data.ReadProjectFile(self.data.ProjectFile)
        #except:
        ##    msg = 'There was an error loading %s.\nPlease make sure it is in the correct format' % fileName
        #    QMessageBox.about(self,title,msg)
	self.main.loadProj(self.data)

    def save(self):
        #print self.filename
        if self.filename.isEmpty():
            self.saveAs()
            return
        #try:
        #    f = open(createString(self.filename),'w+')
        #except:
        #    msg = 'Unable to write to %s.' % self.filename
        #    QMessageBox.about(self,title,msg)
        #    self.saveAs()
        #    return
        self.main.updateProj(self.data)
        self.data.ProjectFile=createString(self.filename)
	try:
	    savefile=createString(self.filename)
	    self.data.SaveProjectFile(savefile)
        except:
	    msg = 'Unable to write to %s.' % self.filename
            QMessageBox.about(self,title,msg)
            self.saveAs()
            return
	#f.close()
        self.main.setEdited(FALSE)

    def saveAs(self):
        fn = QFileDialog.getSaveFileName(QString.null,QString.null,self)
        if not fn.isEmpty():
            self.filename = fn
            self.save()
        else:
            pass
        
    def closeEvent(self,ce):
        if not self.main.edited():
            ce.accept()
            return

        rc = QMessageBox.information(self,title,
                        'Your data has been changed since the last save.',
                    'Save Now','Cancel','Leave Anyway',0,1)
        if rc == 0:
            self.save()
            ce.accept()
        elif rc == 2:
            ce.accept()
        else:
            ce.ignore()

    def about(self):
        QMessageBox.about(self,title,
            'For the latest documentation please visit: \n\nhttp://ncmi.bcm.tmc.edu/~wjiang/savr\n')

    def aboutQt(self):
        QMessageBox.aboutQt(self,'Qt Application Example')



def main():
    if len(sys.argv)>2:
	out="\n USAGE: %s <parameter file> \n" % (sys.argv[0])
	print out
	sys.exit(1)

    global SAVRDIR
    SAVRDIR=os.path.split(sys.argv[0])[0]
    print 'SAVRDIR:', SAVRDIR
    if SAVRDIR=="":
	SAVRDIR='.'

    a = QApplication(sys.argv)
    mw = ApplicationWindow(sys.argv)
    mw.setCaption(title)

    splash=QLabel(None,"SavrSplashScreen",Qt.WStyle_Customize or Qt.WStyle_DialogBorder or Qt.WStyle_tool)
    #splash=QLabel(None,"SavrSplashScreen", Qt.WStyle_Customize or Qt.WStyle_tool)
    splash.setMaximumSize(600,200)
    
    #Set Movie
    MOVIEFILE = SAVRDIR+'/savr.gif'
    print 'MOVIEFILE:',MOVIEFILE
    movie = QMovie( MOVIEFILE )
    movie.step(59)
    splash.setMovie(movie)


    #Set up timing for splash screen
    TIMERLENGTH = 2000 #in milliseconds
    splashTimer = QTimer(splash)
    splashTimer2=QTimer(mw)
    splash.connect(splashTimer,SIGNAL('timeout()'), splash.close)
    mw.connect(splashTimer2,SIGNAL('timeout()'),mw.show)
    splashTimer2.start(TIMERLENGTH-100,TRUE)  #Making this timer a little shorter means that the main window
					      #will load before the splash screen quits, needed 100 on windows
    splashTimer.start(TIMERLENGTH,TRUE)
    splash.show()

    a.connect(a, SIGNAL('lastWindowClosed()'), a, SLOT('quit()'))
    a.exec_loop()


if __name__ == '__main__':
	main()		


